@@ -31,20 +31,16 @@
 # WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 # NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 # SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-from __future__ import division, print_function, absolute_import
-
 import numpy as np
 from numpy.testing import assert_allclose, assert_equal, assert_, assert_warns
 import pytest
 from pytest import raises as assert_raises
 
-from scipy._lib.six import xrange, u
-
 import scipy.cluster.hierarchy
 from scipy.cluster.hierarchy import (
     ClusterWarning, linkage, from_mlab_linkage, to_mlab_linkage,
     num_obs_linkage, inconsistent, cophenet, fclusterdata, fcluster,
-    is_isomorphic, single, leaders, complete, weighted, centroid,
+    is_isomorphic, single, leaders,
     correspond, is_monotonic, maxdists, maxinconsts, maxRstat,
     is_valid_linkage, is_valid_im, to_tree, leaves_list, dendrogram,
     set_link_color_palette, cut_tree, optimal_leaf_ordering,
@@ -58,17 +54,17 @@ from . import hierarchy_test_data
 # Matplotlib is not a scipy dependency but is optionally used in dendrogram, so
 # check if it's available
 try:
-    import matplotlib
+    import matplotlib  # type: ignore[import]
     # and set the backend to be Agg (no gui)
     matplotlib.use('Agg')
     # before importing pyplot
-    import matplotlib.pyplot as plt
+    import matplotlib.pyplot as plt  # type: ignore[import]
     have_matplotlib = True
-except:
+except Exception:
     have_matplotlib = False
 
 
-class TestLinkage(object):
+class TestLinkage:
     def test_linkage_non_finite_elements_in_distance_matrix(self):
         # Tests linkage(Y) where Y contains a non-finite element (e.g. NaN or Inf).
         # Exception expected.
@@ -82,7 +78,7 @@ class TestLinkage(object):
         assert_raises(ValueError, linkage, y)
 
     def test_linkage_tdist(self):
-        for method in ['single', 'complete', 'average', 'weighted', u('single')]:
+        for method in ['single', 'complete', 'average', 'weighted']:
             self.check_linkage_tdist(method)
 
     def check_linkage_tdist(self, method):
@@ -123,7 +119,7 @@ class TestLinkage(object):
         assert_allclose(Z, expectedZ, atol=1e-10)
 
 
-class TestLinkageTies(object):
+class TestLinkageTies:
     _expectations = {
         'single': np.array([[0, 1, 1.41421356, 2],
                             [2, 3, 1.41421356, 3]]),
@@ -152,7 +148,7 @@ class TestLinkageTies(object):
         assert_allclose(Z, expectedZ, atol=1e-06)
 
 
-class TestInconsistent(object):
+class TestInconsistent:
     def test_inconsistent_tdist(self):
         for depth in hierarchy_test_data.inconsistent_ytdist:
             self.check_inconsistent_tdist(depth)
@@ -163,7 +159,7 @@ class TestInconsistent(object):
                         hierarchy_test_data.inconsistent_ytdist[depth])
 
 
-class TestCopheneticDistance(object):
+class TestCopheneticDistance:
     def test_linkage_cophenet_tdist_Z(self):
         # Tests cophenet(Z) on tdist data set.
         expectedM = np.array([268, 295, 255, 255, 295, 295, 268, 268, 295, 295,
@@ -183,7 +179,7 @@ class TestCopheneticDistance(object):
         assert_allclose(M, expectedM, atol=1e-10)
 
 
-class TestMLabLinkageConversion(object):
+class TestMLabLinkageConversion:
     def test_mlab_linkage_conversion_empty(self):
         # Tests from/to_mlab_linkage on empty linkage array.
         X = np.asarray([])
@@ -211,7 +207,7 @@ class TestMLabLinkageConversion(object):
         assert_equal(to_mlab_linkage(Z), Zm)
 
 
-class TestFcluster(object):
+class TestFcluster:
     def test_fclusterdata(self):
         for t in hierarchy_test_data.fcluster_inconsistent:
             self.check_fclusterdata(t, 'inconsistent')
@@ -261,7 +257,7 @@ class TestFcluster(object):
         assert_(is_isomorphic(T, expectedT))
 
 
-class TestLeaders(object):
+class TestLeaders:
     def test_leaders_single(self):
         # Tests leaders using a flat clustering generated by single linkage.
         X = hierarchy_test_data.Q_X
@@ -273,7 +269,7 @@ class TestLeaders(object):
         assert_equal(L, Lright)
 
 
-class TestIsIsomorphic(object):
+class TestIsIsomorphic:
     def test_is_isomorphic_1(self):
         # Tests is_isomorphic on test case #1 (one flat cluster, different labellings)
         a = [1, 1, 1]
@@ -327,17 +323,17 @@ class TestIsIsomorphic(object):
         # nonisomorphic.)
         for nc in [2, 3, 5]:
             self.help_is_isomorphic_randperm(1000, nc, True, 5)
-            
+
     def test_is_isomorphic_7(self):
         # Regression test for gh-6271
         assert_(not is_isomorphic([1, 2, 3], [1, 1, 1]))
-    
+
     def help_is_isomorphic_randperm(self, nobs, nclusters, noniso=False, nerrors=0):
         for k in range(3):
             a = np.int_(np.random.rand(nobs) * nclusters)
             b = np.zeros(a.size, dtype=np.int_)
             P = np.random.permutation(nclusters)
-            for i in xrange(0, a.shape[0]):
+            for i in range(0, a.shape[0]):
                 b[i] = P[a[i]]
             if noniso:
                 Q = np.random.permutation(nobs)
@@ -347,7 +343,7 @@ class TestIsIsomorphic(object):
             assert_(is_isomorphic(b, a) == (not noniso))
 
 
-class TestIsValidLinkage(object):
+class TestIsValidLinkage:
     def test_is_valid_linkage_various_size(self):
         for nrow, ncol, valid in [(2, 5, False), (2, 3, False),
                                   (1, 4, True), (2, 4, True)]:
@@ -378,7 +374,7 @@ class TestIsValidLinkage(object):
     def test_is_valid_linkage_4_and_up(self):
         # Tests is_valid_linkage(Z) on linkage on observation sets between
         # sizes 4 and 15 (step size 3).
-        for i in xrange(4, 15, 3):
+        for i in range(4, 15, 3):
             y = np.random.rand(i*(i-1)//2)
             Z = linkage(y)
             assert_(is_valid_linkage(Z) == True)
@@ -386,7 +382,7 @@ class TestIsValidLinkage(object):
     def test_is_valid_linkage_4_and_up_neg_index_left(self):
         # Tests is_valid_linkage(Z) on linkage on observation sets between
         # sizes 4 and 15 (step size 3) with negative indices (left).
-        for i in xrange(4, 15, 3):
+        for i in range(4, 15, 3):
             y = np.random.rand(i*(i-1)//2)
             Z = linkage(y)
             Z[i//2,0] = -2
@@ -396,7 +392,7 @@ class TestIsValidLinkage(object):
     def test_is_valid_linkage_4_and_up_neg_index_right(self):
         # Tests is_valid_linkage(Z) on linkage on observation sets between
         # sizes 4 and 15 (step size 3) with negative indices (right).
-        for i in xrange(4, 15, 3):
+        for i in range(4, 15, 3):
             y = np.random.rand(i*(i-1)//2)
             Z = linkage(y)
             Z[i//2,1] = -2
@@ -406,7 +402,7 @@ class TestIsValidLinkage(object):
     def test_is_valid_linkage_4_and_up_neg_dist(self):
         # Tests is_valid_linkage(Z) on linkage on observation sets between
         # sizes 4 and 15 (step size 3) with negative distances.
-        for i in xrange(4, 15, 3):
+        for i in range(4, 15, 3):
             y = np.random.rand(i*(i-1)//2)
             Z = linkage(y)
             Z[i//2,2] = -0.5
@@ -416,7 +412,7 @@ class TestIsValidLinkage(object):
     def test_is_valid_linkage_4_and_up_neg_counts(self):
         # Tests is_valid_linkage(Z) on linkage on observation sets between
         # sizes 4 and 15 (step size 3) with negative counts.
-        for i in xrange(4, 15, 3):
+        for i in range(4, 15, 3):
             y = np.random.rand(i*(i-1)//2)
             Z = linkage(y)
             Z[i//2,3] = -2
@@ -424,7 +420,7 @@ class TestIsValidLinkage(object):
             assert_raises(ValueError, is_valid_linkage, Z, throw=True)
 
 
-class TestIsValidInconsistent(object):
+class TestIsValidInconsistent:
     def test_is_valid_im_int_type(self):
         # Tests is_valid_im(R) with integer type.
         R = np.asarray([[0, 1, 3.0, 2],
@@ -455,7 +451,7 @@ class TestIsValidInconsistent(object):
     def test_is_valid_im_4_and_up(self):
         # Tests is_valid_im(R) on im on observation sets between sizes 4 and 15
         # (step size 3).
-        for i in xrange(4, 15, 3):
+        for i in range(4, 15, 3):
             y = np.random.rand(i*(i-1)//2)
             Z = linkage(y)
             R = inconsistent(Z)
@@ -464,7 +460,7 @@ class TestIsValidInconsistent(object):
     def test_is_valid_im_4_and_up_neg_index_left(self):
         # Tests is_valid_im(R) on im on observation sets between sizes 4 and 15
         # (step size 3) with negative link height means.
-        for i in xrange(4, 15, 3):
+        for i in range(4, 15, 3):
             y = np.random.rand(i*(i-1)//2)
             Z = linkage(y)
             R = inconsistent(Z)
@@ -475,7 +471,7 @@ class TestIsValidInconsistent(object):
     def test_is_valid_im_4_and_up_neg_index_right(self):
         # Tests is_valid_im(R) on im on observation sets between sizes 4 and 15
         # (step size 3) with negative link height standard deviations.
-        for i in xrange(4, 15, 3):
+        for i in range(4, 15, 3):
             y = np.random.rand(i*(i-1)//2)
             Z = linkage(y)
             R = inconsistent(Z)
@@ -486,7 +482,7 @@ class TestIsValidInconsistent(object):
     def test_is_valid_im_4_and_up_neg_dist(self):
         # Tests is_valid_im(R) on im on observation sets between sizes 4 and 15
         # (step size 3) with negative link counts.
-        for i in xrange(4, 15, 3):
+        for i in range(4, 15, 3):
             y = np.random.rand(i*(i-1)//2)
             Z = linkage(y)
             R = inconsistent(Z)
@@ -495,7 +491,7 @@ class TestIsValidInconsistent(object):
             assert_raises(ValueError, is_valid_im, R, throw=True)
 
 
-class TestNumObsLinkage(object):
+class TestNumObsLinkage:
     def test_num_obs_linkage_empty(self):
         # Tests num_obs_linkage(Z) with empty linkage.
         Z = np.zeros((0, 4), dtype=np.double)
@@ -515,13 +511,13 @@ class TestNumObsLinkage(object):
     def test_num_obs_linkage_4_and_up(self):
         # Tests num_obs_linkage(Z) on linkage on observation sets between sizes
         # 4 and 15 (step size 3).
-        for i in xrange(4, 15, 3):
+        for i in range(4, 15, 3):
             y = np.random.rand(i*(i-1)//2)
             Z = linkage(y)
             assert_equal(num_obs_linkage(Z), i)
 
 
-class TestLeavesList(object):
+class TestLeavesList:
     def test_leaves_list_1x4(self):
         # Tests leaves_list(Z) on a 1x4 linkage.
         Z = np.asarray([[0, 1, 3.0, 2]], dtype=np.double)
@@ -556,7 +552,7 @@ class TestLeavesList(object):
                                         + node.get_right().pre_order()))
 
 
-class TestCorrespond(object):
+class TestCorrespond:
     def test_correspond_empty(self):
         # Tests correspond(Z, y) with empty linkage and condensed distance matrix.
         y = np.zeros((0,))
@@ -566,18 +562,18 @@ class TestCorrespond(object):
     def test_correspond_2_and_up(self):
         # Tests correspond(Z, y) on linkage and CDMs over observation sets of
         # different sizes.
-        for i in xrange(2, 4):
+        for i in range(2, 4):
             y = np.random.rand(i*(i-1)//2)
             Z = linkage(y)
             assert_(correspond(Z, y))
-        for i in xrange(4, 15, 3):
+        for i in range(4, 15, 3):
             y = np.random.rand(i*(i-1)//2)
             Z = linkage(y)
             assert_(correspond(Z, y))
 
     def test_correspond_4_and_up(self):
         # Tests correspond(Z, y) on linkage and CDMs over observation sets of
-        # different sizes. Correspondance should be false.
+        # different sizes. Correspondence should be false.
         for (i, j) in (list(zip(list(range(2, 4)), list(range(3, 5)))) +
                        list(zip(list(range(3, 5)), list(range(2, 4))))):
             y = np.random.rand(i*(i-1)//2)
@@ -589,7 +585,7 @@ class TestCorrespond(object):
 
     def test_correspond_4_and_up_2(self):
         # Tests correspond(Z, y) on linkage and CDMs over observation sets of
-        # different sizes. Correspondance should be false.
+        # different sizes. Correspondence should be false.
         for (i, j) in (list(zip(list(range(2, 7)), list(range(16, 21)))) +
                        list(zip(list(range(2, 7)), list(range(16, 21))))):
             y = np.random.rand(i*(i-1)//2)
@@ -601,14 +597,14 @@ class TestCorrespond(object):
 
     def test_num_obs_linkage_multi_matrix(self):
         # Tests num_obs_linkage with observation matrices of multiple sizes.
-        for n in xrange(2, 10):
+        for n in range(2, 10):
             X = np.random.rand(n, 4)
             Y = pdist(X)
             Z = linkage(Y)
             assert_equal(num_obs_linkage(Z), n)
 
 
-class TestIsMonotonic(object):
+class TestIsMonotonic:
     def test_is_monotonic_empty(self):
         # Tests is_monotonic(Z) on an empty linkage.
         Z = np.zeros((0, 4))
@@ -680,7 +676,7 @@ class TestIsMonotonic(object):
         assert_equal(is_monotonic(Z), True)
 
 
-class TestMaxDists(object):
+class TestMaxDists:
     def test_maxdists_empty_linkage(self):
         # Tests maxdists(Z) on empty linkage. Expecting exception.
         Z = np.zeros((0, 4), dtype=np.double)
@@ -706,7 +702,7 @@ class TestMaxDists(object):
         assert_allclose(MD, expectedMD, atol=1e-15)
 
 
-class TestMaxInconsts(object):
+class TestMaxInconsts:
     def test_maxinconsts_empty_linkage(self):
         # Tests maxinconsts(Z, R) on empty linkage. Expecting exception.
         Z = np.zeros((0, 4), dtype=np.double)
@@ -742,7 +738,7 @@ class TestMaxInconsts(object):
         assert_allclose(MD, expectedMD, atol=1e-15)
 
 
-class TestMaxRStat(object):
+class TestMaxRStat:
     def test_maxRstat_invalid_index(self):
         for i in [3.3, -1, 4]:
             self.check_maxRstat_invalid_index(i)
@@ -804,7 +800,7 @@ class TestMaxRStat(object):
         assert_allclose(MD, expectedMD, atol=1e-15)
 
 
-class TestDendrogram(object):
+class TestDendrogram:
     def test_dendrogram_single_linkage_tdist(self):
         # Tests dendrogram calculation on single linkage of the tdist data set.
         Z = linkage(hierarchy_test_data.ytdist, 'single')
@@ -816,6 +812,34 @@ class TestDendrogram(object):
         Z = linkage(hierarchy_test_data.ytdist, 'single')
         assert_raises(ValueError, dendrogram, Z, orientation="foo")
 
+    def test_labels_as_array_or_list(self):
+        # test for gh-12418
+        Z = linkage(hierarchy_test_data.ytdist, 'single')
+        labels = np.array([1, 3, 2, 6, 4, 5])
+        result1 = dendrogram(Z, labels=labels, no_plot=True)
+        result2 = dendrogram(Z, labels=labels.tolist(), no_plot=True)
+        assert result1 == result2
+
+    @pytest.mark.skipif(not have_matplotlib, reason="no matplotlib")
+    def test_valid_label_size(self):
+        link = np.array([
+            [0, 1, 1.0, 4],
+            [2, 3, 1.0, 5],
+            [4, 5, 2.0, 6],
+        ])
+        plt.figure()
+        with pytest.raises(ValueError) as exc_info:
+            dendrogram(link, labels=list(range(100)))
+        assert "Dimensions of Z and labels must be consistent."\
+               in str(exc_info.value)
+
+        with pytest.raises(
+                ValueError,
+                match="Dimensions of Z and labels must be consistent."):
+            dendrogram(link, labels=[])
+
+        plt.close()
+
     @pytest.mark.skipif(not have_matplotlib, reason="no matplotlib")
     def test_dendrogram_plot(self):
         for orientation in ['top', 'bottom', 'left', 'right']:
@@ -824,7 +848,7 @@ class TestDendrogram(object):
     def check_dendrogram_plot(self, orientation):
         # Tests dendrogram plotting.
         Z = linkage(hierarchy_test_data.ytdist, 'single')
-        expected = {'color_list': ['g', 'b', 'b', 'b', 'b'],
+        expected = {'color_list': ['C1', 'C0', 'C0', 'C0', 'C0'],
                     'dcoord': [[0.0, 138.0, 138.0, 0.0],
                                [0.0, 219.0, 219.0, 0.0],
                                [0.0, 255.0, 255.0, 219.0],
@@ -836,7 +860,9 @@ class TestDendrogram(object):
                                [25.0, 25.0, 42.5, 42.5],
                                [10.0, 10.0, 33.75, 33.75]],
                     'ivl': ['2', '5', '1', '0', '3', '4'],
-                    'leaves': [2, 5, 1, 0, 3, 4]}
+                    'leaves': [2, 5, 1, 0, 3, 4],
+                    'leaves_color_list': ['C1', 'C1', 'C0', 'C0', 'C0', 'C0'],
+                    }
 
         fig = plt.figure()
         ax = fig.add_subplot(221)
@@ -847,8 +873,8 @@ class TestDendrogram(object):
 
         # test that dendrogram accepts and handle the leaf_font_size and
         # leaf_rotation keywords
-        R1a = dendrogram(Z, ax=ax, orientation=orientation,
-                         leaf_font_size=20, leaf_rotation=90)
+        dendrogram(Z, ax=ax, orientation=orientation,
+                   leaf_font_size=20, leaf_rotation=90)
         testlabel = (
             ax.get_xticklabels()[0]
             if orientation in ['top', 'bottom']
@@ -856,16 +882,16 @@ class TestDendrogram(object):
         )
         assert_equal(testlabel.get_rotation(), 90)
         assert_equal(testlabel.get_size(), 20)
-        R1a = dendrogram(Z, ax=ax, orientation=orientation,
-                         leaf_rotation=90)
+        dendrogram(Z, ax=ax, orientation=orientation,
+                   leaf_rotation=90)
         testlabel = (
             ax.get_xticklabels()[0]
             if orientation in ['top', 'bottom']
             else ax.get_yticklabels()[0]
         )
         assert_equal(testlabel.get_rotation(), 90)
-        R1a = dendrogram(Z, ax=ax, orientation=orientation,
-                         leaf_font_size=20)
+        dendrogram(Z, ax=ax, orientation=orientation,
+                   leaf_font_size=20)
         testlabel = (
             ax.get_xticklabels()[0]
             if orientation in ['top', 'bottom']
@@ -885,15 +911,17 @@ class TestDendrogram(object):
 
         R = dendrogram(Z, 2, 'lastp', show_contracted=True)
         plt.close()
-        assert_equal(R, {'color_list': ['b'],
+        assert_equal(R, {'color_list': ['C0'],
                          'dcoord': [[0.0, 295.0, 295.0, 0.0]],
                          'icoord': [[5.0, 5.0, 15.0, 15.0]],
                          'ivl': ['(2)', '(4)'],
-                         'leaves': [6, 9]})
+                         'leaves': [6, 9],
+                         'leaves_color_list': ['C0', 'C0'],
+                         })
 
         R = dendrogram(Z, 2, 'mtica', show_contracted=True)
         plt.close()
-        assert_equal(R, {'color_list': ['g', 'b', 'b', 'b'],
+        assert_equal(R, {'color_list': ['C1', 'C0', 'C0', 'C0'],
                          'dcoord': [[0.0, 138.0, 138.0, 0.0],
                                     [0.0, 255.0, 255.0, 0.0],
                                     [0.0, 268.0, 268.0, 255.0],
@@ -903,7 +931,9 @@ class TestDendrogram(object):
                                     [25.0, 25.0, 40.0, 40.0],
                                     [10.0, 10.0, 32.5, 32.5]],
                          'ivl': ['2', '5', '1', '0', '(2)'],
-                         'leaves': [2, 5, 1, 0, 7]})
+                         'leaves': [2, 5, 1, 0, 7],
+                         'leaves_color_list': ['C1', 'C1', 'C0', 'C0', 'C0'],
+                         })
 
     def test_dendrogram_colors(self):
         # Tests dendrogram plots with alternate colors
@@ -926,7 +956,7 @@ def calculate_maximum_distances(Z):
     n = Z.shape[0] + 1
     B = np.zeros((n-1,))
     q = np.zeros((3,))
-    for i in xrange(0, n - 1):
+    for i in range(0, n - 1):
         q[:] = 0.0
         left = Z[i, 0]
         right = Z[i, 1]
@@ -944,7 +974,7 @@ def calculate_maximum_inconsistencies(Z, R, k=3):
     n = Z.shape[0] + 1
     B = np.zeros((n-1,))
     q = np.zeros((3,))
-    for i in xrange(0, n - 1):
+    for i in range(0, n - 1):
         q[:] = 0.0
         left = Z[i, 0]
         right = Z[i, 1]
@@ -1059,4 +1089,3 @@ def test_Heap():
     pair = heap.get_min()
     assert_equal(pair['key'], 1)
     assert_equal(pair['value'], 10)
-
