@@ -4,15 +4,33 @@ Module to read / write Fortran unformatted sequential files.
 This is in the spirit of code written by Neil Martinsen-Burrell and Joe Zuntz.
 
 """
-from __future__ import division, print_function, absolute_import
-
 import warnings
 import numpy as np
 
-__all__ = ['FortranFile']
+__all__ = ['FortranFile', 'FortranEOFError', 'FortranFormattingError']
 
 
-class FortranFile(object):
+class FortranEOFError(TypeError, IOError):
+    """Indicates that the file ended properly.
+
+    This error descends from TypeError because the code used to raise
+    TypeError (and this was the only way to know that the file had
+    ended) so users might have ``except TypeError:``.
+
+    """
+    pass
+
+
+class FortranFormattingError(TypeError, IOError):
+    """Indicates that the file ended mid-record.
+
+    Descends from TypeError for backward compatibility.
+
+    """
+    pass
+
+
+class FortranFile:
     """
     A file object for unformatted sequential files from Fortran code.
 
@@ -66,11 +84,11 @@ class FortranFile(object):
     >>> print(f.read_ints(np.int32))
     [1 2 3 4 5]
     >>> print(f.read_reals(float).reshape((5,4), order="F"))
-    [[ 0.          0.05263158  0.10526316  0.15789474]
-     [ 0.21052632  0.26315789  0.31578947  0.36842105]
-     [ 0.42105263  0.47368421  0.52631579  0.57894737]
-     [ 0.63157895  0.68421053  0.73684211  0.78947368]
-     [ 0.84210526  0.89473684  0.94736842  1.        ]]
+    [[0.         0.05263158 0.10526316 0.15789474]
+     [0.21052632 0.26315789 0.31578947 0.36842105]
+     [0.42105263 0.47368421 0.52631579 0.57894737]
+     [0.63157895 0.68421053 0.73684211 0.78947368]
+     [0.84210526 0.89473684 0.94736842 1.        ]]
     >>> f.close()
 
     Or, in Fortran::
@@ -105,8 +123,15 @@ class FortranFile(object):
 
         self._header_dtype = header_dtype
 
-    def _read_size(self):
-        return int(np.fromfile(self._fp, dtype=self._header_dtype, count=1))
+    def _read_size(self, eof_ok=False):
+        n = self._header_dtype.itemsize
+        b = self._fp.read(n)
+        if (not b) and eof_ok:
+            raise FortranEOFError("End of file occurred at end of record")
+        elif len(b) < n:
+            raise FortranFormattingError(
+                "End of file in the middle of the record size")
+        return int(np.frombuffer(b, dtype=self._header_dtype, count=1))
 
     def write_record(self, *items):
         """
@@ -153,11 +178,19 @@ class FortranFile(object):
         Returns
         -------
         data : ndarray
-            A one-dimensional array object.
+            A 1-D array object.
+
+        Raises
+        ------
+        FortranEOFError
+            To signal that no further records are available
+        FortranFormattingError
+            To signal that the end of the file was encountered
+            part-way through a record
 
         Notes
         -----
-        If the record contains a multi-dimensional array, you can specify
+        If the record contains a multidimensional array, you can specify
         the size in the dtype. For example::
 
             INTEGER var(5,4)
@@ -170,7 +203,7 @@ class FortranFile(object):
         column major order, so you need to (i) swap the order of dimensions
         when reading and (ii) transpose the resulting array.
 
-        Alternatively, you can read the data as a 1D array and handle the
+        Alternatively, you can read the data as a 1-D array and handle the
         ordering yourself. For example::
 
             read_record('i4').reshape(5, 4, order='F')
@@ -197,7 +230,7 @@ class FortranFile(object):
             a = record[0]
             b = record[1].T
 
-        Numpy also supports a short syntax for this kind of type::
+        NumPy also supports a short syntax for this kind of type::
 
             record = f.read_record('<f4', '(3,3)<i4')
 
@@ -216,7 +249,7 @@ class FortranFile(object):
         elif not dtypes:
             raise ValueError('Must specify at least one dtype')
 
-        first_size = self._read_size()
+        first_size = self._read_size(eof_ok=True)
 
         dtypes = tuple(np.dtype(dtype) for dtype in dtypes)
         block_size = sum(dtype.itemsize for dtype in dtypes)
@@ -236,6 +269,9 @@ class FortranFile(object):
         data = []
         for dtype in dtypes:
             r = np.fromfile(self._fp, dtype=dtype, count=num_blocks)
+            if len(r) != num_blocks:
+                raise FortranFormattingError(
+                    "End of file in the middle of a record")
             if dtype.shape != ():
                 # Squeeze outmost block dimension for array items
                 if num_blocks == 1:
@@ -268,7 +304,7 @@ class FortranFile(object):
         Returns
         -------
         data : ndarray
-            A one-dimensional array object.
+            A 1-D array object.
 
         See Also
         --------
@@ -291,7 +327,7 @@ class FortranFile(object):
         Returns
         -------
         data : ndarray
-            A one-dimensional array object.
+            A 1-D array object.
 
         See Also
         --------
