@@ -1,8 +1,6 @@
 """Test functions for the sparse.linalg.interface module
 """
 
-from __future__ import division, print_function, absolute_import
-
 from functools import partial
 from itertools import product
 import operator
@@ -14,13 +12,10 @@ import numpy as np
 import scipy.sparse as sparse
 
 from scipy.sparse.linalg import interface
+from scipy.sparse.sputils import matrix
 
 
-# Only test matmul operator (A @ B) when available (Python 3.5+)
-TEST_MATMUL = hasattr(operator, 'matmul')
-
-
-class TestLinearOperator(object):
+class TestLinearOperator:
     def setup_method(self):
         self.A = np.array([[1,2,3],
                            [4,5,6]])
@@ -42,6 +37,7 @@ class TestLinearOperator(object):
                         'shape': A.shape,
                         'matvec': lambda x: np.dot(A, x),
                         'rmatvec': lambda x: np.dot(A.T.conj(), x),
+                        'rmatmat': lambda x: np.dot(A.T.conj(), x),
                         'matmat': lambda x: np.dot(A, x)
                     }]
 
@@ -57,27 +53,45 @@ class TestLinearOperator(object):
             assert_equal(A.dot(np.array([1,2,3])), [14,32])
             assert_equal(A.dot(np.array([[1],[2],[3]])), [[14],[32]])
 
-            assert_equal(A.matvec(np.matrix([[1],[2],[3]])), [[14],[32]])
-            assert_equal(A * np.matrix([[1],[2],[3]]), [[14],[32]])
-            assert_equal(A.dot(np.matrix([[1],[2],[3]])), [[14],[32]])
+            assert_equal(A.matvec(matrix([[1],[2],[3]])), [[14],[32]])
+            assert_equal(A * matrix([[1],[2],[3]]), [[14],[32]])
+            assert_equal(A.dot(matrix([[1],[2],[3]])), [[14],[32]])
 
             assert_equal((2*A)*[1,1,1], [12,30])
-            assert_equal((2*A).rmatvec([1,1]), [10, 14, 18])
+            assert_equal((2 * A).rmatvec([1, 1]), [10, 14, 18])
             assert_equal((2*A).H.matvec([1,1]), [10, 14, 18])
             assert_equal((2*A)*[[1],[1],[1]], [[12],[30]])
-            assert_equal((2*A).matmat([[1],[1],[1]]), [[12],[30]])
+            assert_equal((2 * A).matmat([[1], [1], [1]]), [[12], [30]])
             assert_equal((A*2)*[1,1,1], [12,30])
             assert_equal((A*2)*[[1],[1],[1]], [[12],[30]])
             assert_equal((2j*A)*[1,1,1], [12j,30j])
             assert_equal((A+A)*[1,1,1], [12, 30])
-            assert_equal((A+A).rmatvec([1,1]), [10, 14, 18])
+            assert_equal((A + A).rmatvec([1, 1]), [10, 14, 18])
             assert_equal((A+A).H.matvec([1,1]), [10, 14, 18])
             assert_equal((A+A)*[[1],[1],[1]], [[12], [30]])
             assert_equal((A+A).matmat([[1],[1],[1]]), [[12], [30]])
             assert_equal((-A)*[1,1,1], [-6,-15])
             assert_equal((-A)*[[1],[1],[1]], [[-6],[-15]])
             assert_equal((A-A)*[1,1,1], [0,0])
-            assert_equal((A-A)*[[1],[1],[1]], [[0],[0]])
+            assert_equal((A - A) * [[1], [1], [1]], [[0], [0]])
+
+            X = np.array([[1, 2], [3, 4]])
+            # A_asarray = np.array([[1, 2, 3], [4, 5, 6]])
+            assert_equal((2 * A).rmatmat(X), np.dot((2 * self.A).T, X))
+            assert_equal((A * 2).rmatmat(X), np.dot((self.A * 2).T, X))
+            assert_equal((2j * A).rmatmat(X),
+                         np.dot((2j * self.A).T.conj(), X))
+            assert_equal((A * 2j).rmatmat(X),
+                         np.dot((self.A * 2j).T.conj(), X))
+            assert_equal((A + A).rmatmat(X),
+                         np.dot((self.A + self.A).T, X))
+            assert_equal((A + 2j * A).rmatmat(X),
+                         np.dot((self.A + 2j * self.A).T.conj(), X))
+            assert_equal((-A).rmatmat(X), np.dot((-self.A).T, X))
+            assert_equal((A - A).rmatmat(X),
+                         np.dot((self.A - self.A).T, X))
+            assert_equal((2j * A).rmatmat(2j * X),
+                         np.dot((2j * self.A).T.conj(), 2j * X))
 
             z = A+A
             assert_(len(z.args) == 2 and z.args[0] is A and z.args[1] is A)
@@ -91,9 +105,9 @@ class TestLinearOperator(object):
             assert_(isinstance(A.dot(np.array([1,2,3])), np.ndarray))
             assert_(isinstance(A.dot(np.array([[1],[2],[3]])), np.ndarray))
 
-            assert_(isinstance(A.matvec(np.matrix([[1],[2],[3]])), np.ndarray))
-            assert_(isinstance(A * np.matrix([[1],[2],[3]]), np.ndarray))
-            assert_(isinstance(A.dot(np.matrix([[1],[2],[3]])), np.ndarray))
+            assert_(isinstance(A.matvec(matrix([[1],[2],[3]])), np.ndarray))
+            assert_(isinstance(A * matrix([[1],[2],[3]]), np.ndarray))
+            assert_(isinstance(A.dot(matrix([[1],[2],[3]])), np.ndarray))
 
             assert_(isinstance(2*A, interface._ScaledLinearOperator))
             assert_(isinstance(2j*A, interface._ScaledLinearOperator))
@@ -115,13 +129,20 @@ class TestLinearOperator(object):
                                           get_matvecs(self.B)):
             A = interface.LinearOperator(**matvecsA)
             B = interface.LinearOperator(**matvecsB)
+            # AtimesB = np.array([[22, 28], [49, 64]])
+            AtimesB = self.A.dot(self.B)
+            X = np.array([[1, 2], [3, 4]])
+
+            assert_equal((A * B).rmatmat(X), np.dot((AtimesB).T, X))
+            assert_equal((2j * A * B).rmatmat(X),
+                         np.dot((2j * AtimesB).T.conj(), X))
 
             assert_equal((A*B)*[1,1], [50,113])
             assert_equal((A*B)*[[1],[1]], [[50],[113]])
             assert_equal((A*B).matmat([[1],[1]]), [[50],[113]])
 
-            assert_equal((A*B).rmatvec([1,1]), [71,92])
-            assert_equal((A*B).H.matvec([1,1]), [71,92])
+            assert_equal((A * B).rmatvec([1, 1]), [71, 92])
+            assert_equal((A * B).H.matvec([1, 1]), [71, 92])
 
             assert_(isinstance(A*B, interface._ProductLinearOperator))
 
@@ -133,22 +154,25 @@ class TestLinearOperator(object):
 
         for matvecsC in get_matvecs(self.C):
             C = interface.LinearOperator(**matvecsC)
+            X = np.array([[1, 2], [3, 4]])
+
+            assert_equal(C.rmatmat(X), np.dot((self.C).T, X))
+            assert_equal((C**2).rmatmat(X),
+                         np.dot((np.dot(self.C, self.C)).T, X))
 
             assert_equal((C**2)*[1,1], [17,37])
-            assert_equal((C**2).rmatvec([1,1]), [22,32])
-            assert_equal((C**2).H.matvec([1,1]), [22,32])
+            assert_equal((C**2).rmatvec([1, 1]), [22, 32])
+            assert_equal((C**2).H.matvec([1, 1]), [22, 32])
             assert_equal((C**2).matmat([[1],[1]]), [[17],[37]])
 
             assert_(isinstance(C**2, interface._PowerLinearOperator))
 
     def test_matmul(self):
-        if not TEST_MATMUL:
-            pytest.skip("matmul is only tested in Python 3.5+")
-
         D = {'shape': self.A.shape,
              'matvec': lambda x: np.dot(self.A, x).reshape(self.A.shape[0]),
              'rmatvec': lambda x: np.dot(self.A.T.conj(),
                                          x).reshape(self.A.shape[1]),
+             'rmatmat': lambda x: np.dot(self.A.T.conj(), x),
              'matmat': lambda x: np.dot(self.A, x)}
         A = interface.LinearOperator(**D)
         B = np.array([[1, 2, 3],
@@ -162,42 +186,47 @@ class TestLinearOperator(object):
         assert_raises(ValueError, operator.matmul, 2, A)
 
 
-class TestAsLinearOperator(object):
+class TestAsLinearOperator:
     def setup_method(self):
         self.cases = []
 
-        def make_cases(dtype):
-            self.cases.append(np.matrix([[1,2,3],[4,5,6]], dtype=dtype))
-            self.cases.append(np.array([[1,2,3],[4,5,6]], dtype=dtype))
-            self.cases.append(sparse.csr_matrix([[1,2,3],[4,5,6]], dtype=dtype))
+        def make_cases(original, dtype):
+            cases = []
+
+            cases.append((matrix(original, dtype=dtype), original))
+            cases.append((np.array(original, dtype=dtype), original))
+            cases.append((sparse.csr_matrix(original, dtype=dtype), original))
 
             # Test default implementations of _adjoint and _rmatvec, which
             # refer to each other.
             def mv(x, dtype):
-                y = np.array([1 * x[0] + 2 * x[1] + 3 * x[2],
-                              4 * x[0] + 5 * x[1] + 6 * x[2]], dtype=dtype)
+                y = original.dot(x)
                 if len(x.shape) == 2:
                     y = y.reshape(-1, 1)
                 return y
 
             def rmv(x, dtype):
-                return np.array([1 * x[0] + 4 * x[1],
-                                 2 * x[0] + 5 * x[1],
-                                 3 * x[0] + 6 * x[1]], dtype=dtype)
+                return original.T.conj().dot(x)
 
             class BaseMatlike(interface.LinearOperator):
+                args = ()
+
                 def __init__(self, dtype):
                     self.dtype = np.dtype(dtype)
-                    self.shape = (2,3)
+                    self.shape = original.shape
 
                 def _matvec(self, x):
                     return mv(x, self.dtype)
 
             class HasRmatvec(BaseMatlike):
+                args = ()
+
                 def _rmatvec(self,x):
                     return rmv(x, self.dtype)
 
             class HasAdjoint(BaseMatlike):
+                args = ()
+
                 def _adjoint(self):
                     shape = self.shape[1], self.shape[0]
                     matvec = partial(rmv, dtype=self.dtype)
@@ -207,51 +236,88 @@ class TestAsLinearOperator(object):
                                                     dtype=self.dtype,
                                                     shape=shape)
 
-            self.cases.append(HasRmatvec(dtype))
-            self.cases.append(HasAdjoint(dtype))
+            class HasRmatmat(HasRmatvec):
+                def _matmat(self, x):
+                    return original.dot(x)
 
-        make_cases('int32')
-        make_cases('float32')
-        make_cases('float64')
+                def _rmatmat(self, x):
+                    return original.T.conj().dot(x)
+
+            cases.append((HasRmatvec(dtype), original))
+            cases.append((HasAdjoint(dtype), original))
+            cases.append((HasRmatmat(dtype), original))
+            return cases
+
+        original = np.array([[1,2,3], [4,5,6]])
+        self.cases += make_cases(original, np.int32)
+        self.cases += make_cases(original, np.float32)
+        self.cases += make_cases(original, np.float64)
+        self.cases += [(interface.aslinearoperator(M).T, A.T)
+                       for M, A in make_cases(original.T, np.float64)]
+        self.cases += [(interface.aslinearoperator(M).H, A.T.conj())
+                       for M, A in make_cases(original.T, np.float64)]
+
+        original = np.array([[1, 2j, 3j], [4j, 5j, 6]])
+        self.cases += make_cases(original, np.complex_)
+        self.cases += [(interface.aslinearoperator(M).T, A.T)
+                       for M, A in make_cases(original.T, np.complex_)]
+        self.cases += [(interface.aslinearoperator(M).H, A.T.conj())
+                       for M, A in make_cases(original.T, np.complex_)]
 
     def test_basic(self):
 
-        for M in self.cases:
+        for M, A_array in self.cases:
             A = interface.aslinearoperator(M)
             M,N = A.shape
 
-            assert_equal(A.matvec(np.array([1,2,3])), [14,32])
-            assert_equal(A.matvec(np.array([[1],[2],[3]])), [[14],[32]])
+            xs = [np.array([1, 2, 3]),
+                  np.array([[1], [2], [3]])]
+            ys = [np.array([1, 2]), np.array([[1], [2]])]
 
-            assert_equal(A * np.array([1,2,3]), [14,32])
-            assert_equal(A * np.array([[1],[2],[3]]), [[14],[32]])
+            if A.dtype == np.complex_:
+                xs += [np.array([1, 2j, 3j]),
+                       np.array([[1], [2j], [3j]])]
+                ys += [np.array([1, 2j]), np.array([[1], [2j]])]
 
-            assert_equal(A.rmatvec(np.array([1,2])), [9,12,15])
-            assert_equal(A.rmatvec(np.array([[1],[2]])), [[9],[12],[15]])
-            assert_equal(A.H.matvec(np.array([1,2])), [9,12,15])
-            assert_equal(A.H.matvec(np.array([[1],[2]])), [[9],[12],[15]])
+            x2 = np.array([[1, 4], [2, 5], [3, 6]])
 
-            assert_equal(
-                    A.matmat(np.array([[1,4],[2,5],[3,6]])),
-                    [[14,32],[32,77]])
+            for x in xs:
+                assert_equal(A.matvec(x), A_array.dot(x))
+                assert_equal(A * x, A_array.dot(x))
 
-            assert_equal(A * np.array([[1,4],[2,5],[3,6]]), [[14,32],[32,77]])
+            assert_equal(A.matmat(x2), A_array.dot(x2))
+            assert_equal(A * x2, A_array.dot(x2))
+
+            for y in ys:
+                assert_equal(A.rmatvec(y), A_array.T.conj().dot(y))
+                assert_equal(A.T.matvec(y), A_array.T.dot(y))
+                assert_equal(A.H.matvec(y), A_array.T.conj().dot(y))
+
+            for y in ys:
+                if y.ndim < 2:
+                    continue
+                assert_equal(A.rmatmat(y), A_array.T.conj().dot(y))
+                assert_equal(A.T.matmat(y), A_array.T.dot(y))
+                assert_equal(A.H.matmat(y), A_array.T.conj().dot(y))
 
             if hasattr(M,'dtype'):
                 assert_equal(A.dtype, M.dtype)
 
+            assert_(hasattr(A, 'args'))
+
     def test_dot(self):
 
-        for M in self.cases:
+        for M, A_array in self.cases:
             A = interface.aslinearoperator(M)
             M,N = A.shape
 
-            assert_equal(A.dot(np.array([1,2,3])), [14,32])
-            assert_equal(A.dot(np.array([[1],[2],[3]])), [[14],[32]])
+            x0 = np.array([1, 2, 3])
+            x1 = np.array([[1], [2], [3]])
+            x2 = np.array([[1, 4], [2, 5], [3, 6]])
 
-            assert_equal(
-                    A.dot(np.array([[1,4],[2,5],[3,6]])),
-                    [[14,32],[32,77]])
+            assert_equal(A.dot(x0), A_array.dot(x0))
+            assert_equal(A.dot(x1), A_array.dot(x1))
+            assert_equal(A.dot(x2), A_array.dot(x2))
 
 
 def test_repr():
@@ -278,7 +344,7 @@ def test_attributes():
 
     B = interface.LinearOperator(shape=(4, 3), matvec=always_four_ones)
 
-    for op in [A, B, A * B, A.H, A + A, B + B, A ** 4]:
+    for op in [A, B, A * B, A.H, A + A, B + B, A**4]:
         assert_(hasattr(op, "dtype"))
         assert_(hasattr(op, "shape"))
         assert_(hasattr(op, "_matvec"))
@@ -307,7 +373,7 @@ def test_inheritance():
 
     class Identity(interface.LinearOperator):
         def __init__(self, n):
-            super(Identity, self).__init__(dtype=None, shape=(n, n))
+            super().__init__(dtype=None, shape=(n, n))
 
         def _matvec(self, x):
             return x
@@ -318,7 +384,7 @@ def test_inheritance():
 
     class MatmatOnly(interface.LinearOperator):
         def __init__(self, A):
-            super(MatmatOnly, self).__init__(A.dtype, A.shape)
+            super().__init__(A.dtype, A.shape)
             self.A = A
 
         def _matmat(self, x):
@@ -351,7 +417,7 @@ def test_no_double_init():
 
     # It should call matvec exactly once (in order to determine the
     # operator dtype)
-    A = interface.LinearOperator((2, 2), matvec=matvec)
+    interface.LinearOperator((2, 2), matvec=matvec)
     assert_equal(call_count[0], 1)
 
 def test_adjoint_conjugate():
@@ -365,3 +431,20 @@ def test_adjoint_conjugate():
 
     assert_equal(B.dot(v), Y.dot(v))
     assert_equal(B.H.dot(v), Y.T.conj().dot(v))
+
+def test_ndim():
+    X = np.array([[1]])
+    A = interface.aslinearoperator(X)
+    assert_equal(A.ndim, 2)
+
+def test_transpose_noconjugate():
+    X = np.array([[1j]])
+    A = interface.aslinearoperator(X)
+
+    B = 1j * A
+    Y = 1j * X
+
+    v = np.array([1])
+
+    assert_equal(B.dot(v), Y.dot(v))
+    assert_equal(B.T.dot(v), Y.T.dot(v))
