@@ -1,18 +1,10 @@
-from __future__ import division, print_function, absolute_import
-
-from scipy._lib._numpy_compat import suppress_warnings
-
 try:
     import mpmath as mp
 except ImportError:
     pass
 
 try:
-    # Can remove when sympy #11255 is resolved; see
-    # https://github.com/sympy/sympy/issues/11255
-    with suppress_warnings() as sup:
-        sup.filter(DeprecationWarning, "inspect.getargspec.. is deprecated")
-        from sympy.abc import x
+    from sympy.abc import x  # type: ignore[import]
 except ImportError:
     pass
 
